import { async as subject } from 'most-subject';
import { effectFabric, eventFabric } from './store/fabric';
import { createIDType } from './id';
var nextID = createIDType();
export class Domain {
  constructor() {
    this.id = nextID();
    this.update$ = subject();
    this.state$ = this.update$.map(({
      state
    }) => state).multicast();

    this.dispatch$ = (() => {
      var subj = subject();
      subj.skipRepeats().observe(e => {
        if (isAction(e)) this.dispatch(e);
      });
      return subj;
    })();
  }

  *scope() {
    yield* this.scopeName;
    yield this.domainName;
  }

  getState() {
    return this.state$;
  }

  mergeEvents(emitter) {
    emitter.event$ = this.dispatch$;
  }

  effect(description) {
    var result = effectFabric(description, this);
    return result;
  }

  event(description) {
    var result = eventFabric(description, this); // this.mergeEvents(result)

    return result;
  }

  connect(domain) {
    domain.dispatch = this.dispatch;
    domain.stateGetter = this.stateGetter;
    domain.reduxSubscribe = this.reduxSubscribe;
    domain.dispatch$ = this.dispatch$;
    domain.scopeName = [...this.scope(), domain.domainName];
    domain.injector = this.injector; // domain.getState = this.stateGetter

    domain.subscribe = this.subscribe;
    domain.replaceReducer = this.replaceReducer;
  }

}
export function createDomain(name) {
  var result = new Domain();
  result.domainName = name;
  return result;
}

function isAction(value)
/*::%checks*/
{
  return typeof value === 'object' && value != null && typeof value.type === 'string' && value.type.length > 0;
}
//# sourceMappingURL=domain.js.map