"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDomain = createDomain;
exports.Domain = void 0;

var _mostSubject = require("most-subject");

var _fabric = require("./store/fabric");

var _id = require("./id");

var nextID = (0, _id.createIDType)();

class Domain {
  constructor() {
    this.id = nextID();
    this.update$ = (0, _mostSubject.async)();
    this.state$ = this.update$.map(({
      state
    }) => state).multicast();

    this.dispatch$ = (() => {
      var subj = (0, _mostSubject.async)();
      subj.skipRepeats().observe(e => {
        if (isAction(e)) this.dispatch(e);
      });
      return subj;
    })();
  }

  *scope() {
    yield* this.scopeName;
    yield this.domainName;
  }

  getState() {
    return this.state$;
  }

  mergeEvents(emitter) {
    emitter.event$ = this.dispatch$;
  }

  effect(description) {
    var result = (0, _fabric.effectFabric)(description, this);
    return result;
  }

  event(description) {
    var result = (0, _fabric.eventFabric)(description, this); // this.mergeEvents(result)

    return result;
  }

  connect(domain) {
    domain.dispatch = this.dispatch;
    domain.stateGetter = this.stateGetter;
    domain.reduxSubscribe = this.reduxSubscribe;
    domain.dispatch$ = this.dispatch$;
    domain.scopeName = [...this.scope(), domain.domainName];
    domain.injector = this.injector; // domain.getState = this.stateGetter

    domain.subscribe = this.subscribe;
    domain.replaceReducer = this.replaceReducer;
  }

}

exports.Domain = Domain;

function createDomain(name) {
  var result = new Domain();
  result.domainName = name;
  return result;
}

function isAction(value)
/*::%checks*/
{
  return typeof value === 'object' && value != null && typeof value.type === 'string' && value.type.length > 0;
}
//# sourceMappingURL=domain.js.map